package ru.ifmo.hydro;

import com.evelopers.unimod.core.stateworks.Event;
import com.evelopers.unimod.runtime.ControlledObject;
import com.evelopers.unimod.runtime.ModelEngine;
import com.evelopers.unimod.runtime.context.StateMachineContext;
import com.evelopers.unimod.runtime.context.StateMachineContextImpl;



public class Hydro implements ControlledObject, HydroHandle {

    public static final int MAX_ROTATIONS = 2000;
    public static final int MIN_ROTATIONS = 1000;
    public static final int ROTATION_COUNT_PER_TIME = 10;

    private  ModelEngine engine;

    private static Hydro o1;

    private HydroVisualizer hydroVisualizer;

    private volatile int rotations = 0;
    public volatile int gear = 0;
    private volatile boolean gasPedalPressed;
    private volatile boolean switchedOn;

    private CustomThread customThread;

    public Hydro() {
        System.out.println("Default constructor");
    }

    private Hydro(ModelEngine engine) {
        System.out.println("Constructor");
        this.engine = engine;
        hydroVisualizer = new HydroVisualizerForm("  ", this);
        hydroVisualizer.setStatus(0);
        customThread = new CustomThread();
    }

    public static void init(ModelEngine engine){
        o1 = new Hydro(engine);
    }

    private void lowGear() {
        notify(CustomEventProvider.e2);        
    }

    private void upGear() {
        notify(CustomEventProvider.e1);
    }

    private void checkRotations() {
        if (o1.rotations >= MAX_ROTATIONS) {
            if (o1.gear > 0 && o1.gear < 5) {
                o1.upGear();
            }
        } else if (o1.rotations <= MIN_ROTATIONS) {            
            if (o1.gear > 1) {
                o1.lowGear();
            }
        }
        if (o1.rotations < MIN_ROTATIONS) {
            o1.rotations = MIN_ROTATIONS;
        }
        if (o1.rotations > MAX_ROTATIONS) {
            o1.rotations = MAX_ROTATIONS;
        }
    }

    public void notify(String event){
        o1.engine.getEventManager().handle(new Event(event), StateMachineContextImpl.create());
    }

    public void start() {
        o1.notify(CustomEventProvider.e7);        
    }

    public void stop() {
        o1.notify(CustomEventProvider.e6);
    }

    public void gasPedalPressed() {
        o1.gasPedalPressed = true;
    }

    public void gasPedalReleased() {
        o1.gasPedalPressed = false;
    }

    public void switchToForward() {
        notify(CustomEventProvider.e3);
    }

    public void switchToBackward() {
        notify(CustomEventProvider.e4);
    }

    public void switchToNeutral() {
        notify(CustomEventProvider.e5);
    }

    /**
     * @unimod.action.descr is something correct
     */
    public boolean z3(StateMachineContext context) {
        o1.gear = 1;
        o1.hydroVisualizer.setGear(o1.gear);
        return true;
    }

    /**
     * @unimod.action.descr is something correct
     */
    public boolean z4(StateMachineContext context) {
        o1.gear = 2;
        o1.hydroVisualizer.setGear(o1.gear);
        return true;
    }

    /**
     * @unimod.action.descr is something correct
     */
    public boolean z5(StateMachineContext context) {
        o1.gear = 3;
        o1.hydroVisualizer.setGear(o1.gear);
        return true;
    }

    /**
     * @unimod.action.descr is something correct
     */
    public boolean z6(StateMachineContext context) {
        o1.gear = 4;
        o1.hydroVisualizer.setGear(o1.gear);
        return true;
    }

    /**
     * @unimod.action.descr is something correct
     */
    public boolean z7(StateMachineContext context) {
        o1.gear = 5;
        o1.hydroVisualizer.setGear(o1.gear);
        return true;
    }

    /**
     * @unimod.action.descr is something correct
     */
    public boolean z8(StateMachineContext context) {
        o1.gear = -1;
        o1.hydroVisualizer.setGear(o1.gear);
        return true;
    }

    /**
     * @unimod.action.descr Starting the engine
     */
    public boolean z1(StateMachineContext context) {
        o1.switchedOn = true;
        o1.rotations = MIN_ROTATIONS;
        o1.gear = 0;
        o1.hydroVisualizer.switchOn();
        o1.hydroVisualizer.setGear(o1.gear);
        o1.hydroVisualizer.setStatus(1);
        o1.hydroVisualizer.setStatus(3);
        if (!o1.customThread.isAlive()) {
            o1.customThread.start();
        }
        return false;
    }

    /**
     * @unimod.action.descr Stopping the engine
     */
    public boolean z2(StateMachineContext context) {
        o1.switchedOn = false;
        o1.rotations = 0;
        o1.hydroVisualizer.setRotations(o1.rotations);
        o1.hydroVisualizer.setStatus(0);
        System.exit(0);
        return false;
    }

    public class CustomThread extends Thread {
        public void run() {
            System.out.println("In the customThread.run");
            while (o1.switchedOn) {
                if (o1.gasPedalPressed) {
                	if (o1.rotations < MAX_ROTATIONS) {
                		o1.rotations += ROTATION_COUNT_PER_TIME;
                	}
                } else {
                	if (o1.rotations > MIN_ROTATIONS) {
                		o1.rotations -= ROTATION_COUNT_PER_TIME;
                	}
                }
                if (o1.rotations < MIN_ROTATIONS) {
                    o1.rotations = MIN_ROTATIONS;
                }
                if (o1.gear != 0) {
                    checkRotations();
                }
                o1.hydroVisualizer.setRotations(o1.rotations);
                try {
                    Thread.sleep(10);
                } catch (InterruptedException e) {
                    e.printStackTrace();  
                }
            }

        }
    }

    /**
     * @unimod.action.descr Low rotations to MIN_ROTATIONS value
     */
    public boolean z9(StateMachineContext context) {
        o1.rotations = MIN_ROTATIONS;
        return true;
    }

    /**
     * @unimod.action.descr Up rotations to MAX_ROTATIONS value
     */
    public boolean z10(StateMachineContext context) {
        o1.rotations = MAX_ROTATIONS;
        return true;
    }
}