package ru.ifmo.hydro;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

public class HydroVisualizerForm extends JFrame implements HydroVisualizer, ActionListener, MouseListener {
    public static final long serialVersionUID = 123456789052l;
    public static final String OFF = "";
    public static final String ON = "";
    public static final String FORWARD = "";
    public static final String BACKWARD = "";
    public static final String NEUTRAL = "";
    public static final String GAS = "";
    public static final String HYDRO_STATE = "";

    private JButton btnRotations;
    private JButton btnOn;
    private JButton btnOff;
    private JRadioButton rbnForward;
    private JRadioButton rbnBackward;
    private JRadioButton rbnNeutral;
    private TachometerPanel tachometerPanel;

    private Hydro hydro;

    public HydroVisualizerForm(String title, Hydro hydro) {
        super(title);

        this.hydro = hydro;

        setSize(400, 300);
        setResizable(false);
        
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();        
        this.setLocation((screenSize.width - this.getWidth()) / 2, 
                (screenSize.height - this.getHeight()) / 2);

        getContentPane().setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(2, 5, 2, 5);

        c.gridx = 0;
        c.gridy = 0;
        btnOn = new JButton(ON);
        btnOn.addActionListener(this);
        getContentPane().add(btnOn, c);

        c.gridx = 1;
        btnOff = new JButton(OFF);
        btnOff.addActionListener(this);
        getContentPane().add(btnOff, c);

        JPanel switchPanel = new JPanel(new GridBagLayout());
        Border etched = BorderFactory.createEtchedBorder();
        TitledBorder titledBorder = BorderFactory.createTitledBorder(etched, HYDRO_STATE + ":");
        switchPanel.setBorder(titledBorder);

        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth = 2;
        getContentPane().add(switchPanel, c);

        c.gridx = 2;
        c.weightx = 1.0;
        c.fill = GridBagConstraints.BOTH;
        tachometerPanel = new TachometerPanel();
        tachometerPanel.setBorder(BorderFactory.createEtchedBorder());
        getContentPane().add(tachometerPanel, c);


        rbnNeutral = new JRadioButton(NEUTRAL);
        rbnNeutral.setEnabled(false);
        rbnNeutral.addActionListener(this);
        rbnForward = new JRadioButton(FORWARD);
        rbnForward.setEnabled(false);
        rbnForward.addActionListener(this);
        rbnBackward = new JRadioButton(BACKWARD);
        rbnBackward.setEnabled(false);
        rbnBackward.addActionListener(this);
        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add(rbnNeutral);
        buttonGroup.add(rbnForward);
        buttonGroup.add(rbnBackward);

        c.gridx = 0;
        c.weightx = 0.0;
        c.gridy = 0;
        c.anchor = GridBagConstraints.WEST;
        switchPanel.add(rbnForward, c);

        c.gridy = 1;
        switchPanel.add(rbnBackward, c);

        c.gridy = 2;
        switchPanel.add(rbnNeutral, c);

        c.gridy = 2;
        c.gridwidth = 4;
        c.anchor = GridBagConstraints.CENTER;
        c.fill = GridBagConstraints.NONE;
        c.weightx = 1.0;
        btnRotations = new JButton(GAS);
        btnRotations.addMouseListener(this);
        getContentPane().add(btnRotations, c);

        setVisible(true);

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    public void setRotations(int rotations) {
        tachometerPanel.setRotations(rotations);
        repaint();
    }

    public void setGear(int gear){
        tachometerPanel.setGear(gear);
    }

    public void setStatus(int status) {
    	switch (status) {
    	case 3: 
			rbnNeutral.setSelected(true);
			break;    		
    	}
    }
    
    public void switchOn() {
    	rbnNeutral.setEnabled(true);
    	rbnForward.setEnabled(true);
    	rbnBackward.setEnabled(true);
    }

    public void actionPerformed(ActionEvent ae) {
        if (ae.getSource() == btnOn) {
            hydro.start();
        }
        if (ae.getSource() == btnOff) {
            hydro.stop();
        }

        if (ae.getSource() == rbnForward) {
            hydro.switchToForward();
        }
        if (ae.getSource() == rbnBackward) {
            hydro.switchToBackward();
        }
        if (ae.getSource() == rbnNeutral) {
            hydro.switchToNeutral();
        }
    }

    public void mouseClicked(MouseEvent e) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void mousePressed(MouseEvent e) {
        hydro.gasPedalPressed();
    }

    public void mouseReleased(MouseEvent e) {
        hydro.gasPedalReleased();
    }

    public void mouseEntered(MouseEvent e) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void mouseExited(MouseEvent e) {
        //To change body of implemented methods use File | Settings | File Templates.
    }
}